<?php

namespace Drupal\multi_crop_image_widget_crop\Plugin\MultiCrop;

use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Entity\ImageStyle;
use Drupal\multi_crop\MultiCropPluginBase;

/**
 * Plugin implementation of the multi_crop.
 *
 * @MultiCrop(
 *   id = "image_widget_crop",
 *   target_field_name = "image_crop",
 *   label = @Translation("Image Widget Crop"),
 *   description = @Translation("Manage Multicrop for image_widget_crop."),
 *   image_style_effect = {"crop_crop"}
 * )
 */
class ImageWidgetCrop extends MultiCropPluginBase {

  /**
   * {@inheritdoc}
   */
  public function saveCrop($crop_settings, string $image_style, string $old_uri, string $new_uri, int $width, int $height) {

    $style = ImageStyle::load($image_style);

    $style_dependencies = $style->getDependencies();
    foreach ($style_dependencies as $dep) {
      if (strstr($dep[0], 'crop.type')) {
        $croptype = str_replace('crop.type.', '', $dep[0]);
      }
    }

    $crop_created = FALSE;
    // Filter to the concerned crop setting.
    foreach ($crop_settings['crop_wrapper'] as $crop_type => $container) {
      // Get Crop value.
      if ($croptype != $crop_type) {
        continue;
      }

      $crop_values = $container['crop_container']['values'];

      // If crop are set.
      if ($crop_values['crop_applied'] == '1') {

        // Recover Crop.
        $crop = $this->retrievCrop($crop_type, $old_uri, $new_uri);

        // Get Center coordinate of crop zone on original image.
        $axis_coordinate = \Drupal::service('image_widget_crop.manager')->getAxisCoordinates(
          ['x' => $crop_values['x'], 'y' => $crop_values['y']],
          ['width' => $crop_values['width'], 'height' => $crop_values['height']]
        );

        // Set position & size.
        $crop->setPosition($axis_coordinate['x'], $axis_coordinate['y']);
        $crop->setSize($crop_values['width'], $crop_values['height']);

        // Save Crop.
        $crop->save();
        $crop_created = TRUE;
      }
    }

    return $crop_created;
  }

  /**
   * {@inheritdoc}
   */
  public static function processFieldData($field_data) {

    // Filter to the concerned crop setting.
    foreach ($field_data['crop_wrapper'] as $crop_type => $container) {
      // Get Crop value.
      $crop_values = $container['crop_container']['values'];
      if ($crop_values['crop_applied'] == 0) {
        unset($field_data['crop_wrapper'][$crop_type]);
      }
    }

    if (count($field_data['crop_wrapper']) == 0) {
      return NULL;
    }
    else {
      return $field_data;
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getComponentConfig($default_values, $image_style_crops) {
    $component_data = [
      'type' => 'image_widget_crop',
      'settings' => [
        'progess_indicator' => 'throbber',
        'preview_image_style' => 'thumbnail',
        'crop_preview_image_style' => 'crop_thumbnail',
        'crop_list' => $image_style_crops,
        'warn_multiple_usages' => FALSE,
        'show_crop_area' => TRUE,
        'show_default_crop' => TRUE,

      ],
    ];
    return $component_data;
  }

  /**
   * {@inheritdoc}
   */
  public function finishElement(&$form, $source_field_name, $default_values) {
    parent::finishElement($form, $source_field_name, $default_values);

    $form['#attached']['library'][] = 'multi_crop_image_widget_crop/editor_media_dialog_fix';
    $widget = &$form[$source_field_name]['widget'][0];

    if (array_key_exists('data-crop-settings', $default_values) && $default_values['data-crop-settings'] != '') {
      $widget['#default_value']['image_crop']['crop_wrapper'] = @unserialize($default_values['data-crop-settings'], ['allowed_classes' => FALSE]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function widgetSave(array $form, FormStateInterface $form_state) {
    $target = ['field_media_image', 0, 'image_crop', 'crop_wrapper'];
    $settings = $form_state->getValue($target);
    foreach ($settings as $key => $data) {
      unset($settings[$key]['crop_container']['reset']);
    }
    if ($settings) {
      $form_state->setValue(['attributes', 'data-crop-settings'], serialize($settings));
    }

    $form_state->setValue(['attributes', 'data-crop-type'], 'image_widget_crop');
  }

  /**
   * {@inheritdoc}
   */
  public function processEmbedData($embed_settings, array $variables) {

    $settings = unserialize($embed_settings, ['allowed_classes' => FALSE]);

    $crop_settings = [
      'plugin_id' => 'image_widget_crop',
      'crop_setting' => ['crop_wrapper' => $settings],
      'context' => md5($embed_settings),
      'image_style' => $variables['image_style'],
      'base_crop_folder' => 'multi_crop_embed',
    ];

    return $crop_settings;
  }

}
